import java.io.FileOutputStream;

import java.io.IOException;

import java.io.PrintWriter;

import java.util.TreeMap;

import java.util.Iterator;

import java.text.NumberFormat;

import java.util.Locale;



/**

* SymbolCounter

* @author willema

*/

public class SymbolCounter extends TreeMap {

    

    private long symbolCount;

    

    public SymbolCounter() {

        super();

        symbolCount = 0;

    }

    

    public void addSymbol(String symbol) {

        if (this.containsKey(symbol)) {

            Long count = (Long)this.get(symbol);

            this.put(symbol, new Long(count.longValue()+1));

        }

        else {

            this.put(symbol, new Long(1));

        }

        symbolCount++;

    }

    

    public long getSymbolCount(String symbol) {

        if (this.containsKey(symbol)) {

            Long count = (Long)this.get(symbol);

            return count.longValue();

        } else {

            return 0;

        }

    }

    

    public double getSymbolFreq(String symbol) {

        if (this.containsKey(symbol)) {

            Long count = (Long)this.get(symbol);

            return count.doubleValue() / symbolCount;

        } else {

            return 0.0;

        }

    }

    

    public long getTotalCount() {

        return symbolCount;

    }

    

    public boolean writeToFile(String filename, boolean absolute) {

        try {

            NumberFormat f = NumberFormat.getNumberInstance(Locale.ENGLISH);

            f.setMaximumFractionDigits(4);

            f.setMinimumFractionDigits(1);            

            PrintWriter file = new PrintWriter(new FileOutputStream(filename), true);

            Iterator iter = this.keySet().iterator();

            while(iter.hasNext()) {

                String symbol = (String)iter.next();

                file.print(symbol.toString());

                file.print(" ");

                if (absolute) {

                    long value = getSymbolCount(symbol);

                    file.println(value);

                } else {

                    double value = getSymbolFreq(symbol);

                    file.println(f.format(value));

                }

            }

        } catch (IOException e) {

            return false;

        }

        return true;

    }

    

    public boolean writeToFile(String filename, SymbolCounter pre) {

        try {

            NumberFormat f = NumberFormat.getNumberInstance(Locale.ENGLISH);

            f.setMaximumFractionDigits(4);

            f.setMinimumFractionDigits(1);

            PrintWriter file = new PrintWriter(new FileOutputStream(filename), true);

            Iterator iter = this.keySet().iterator();

            while(iter.hasNext()) {

                String symbol = (String)iter.next();

                String prefix = symbol.substring(0,symbol.lastIndexOf('_'));

                file.print(symbol);

                file.print(" ");

                // calculate relative probability p(a|b)=p(ab)/p(b)

                double value = Math.min(getSymbolFreq(symbol) / pre.getSymbolFreq(prefix), 1.0);

                file.println(f.format(value));

            }

        } catch (IOException e) {

            return false;

        }

        return true;

    }

    

    public double getEntropy() {

        Iterator iter = this.keySet().iterator();

        double entropy = 0.0;

        while(iter.hasNext()) {

            String symbol = (String)iter.next();

            double prob = getSymbolFreq(symbol);

            entropy += prob * log2(prob);

        }

        return -entropy;

    }

    

    public double log2(double x) {

        return Math.log(x)/Math.log(2.0);

    }

    

}

